==> pythran-0.5.0.tar.gz: pythran-0.5.0/pythran/nt2/core/functions/common/flipud.hpp <==
//==============================================================================
//         Copyright 2003 - 2011   LASMEA UMR 6602 CNRS/Univ. Clermont II
//         Copyright 2009 - 2011   LRI    UMR 8623 CNRS/Univ Paris Sud XI
//
//          Distributed under the Boost Software License, Version 1.0.
//                 See accompanying file LICENSE.txt or copy at
//                     http://www.boost.org/LICENSE_1_0.txt
//==============================================================================
#ifndef NT2_CORE_FUNCTIONS_COMMON_FLIPUD_HPP_INCLUDED
#define NT2_CORE_FUNCTIONS_COMMON_FLIPUD_HPP_INCLUDED

#include <nt2/core/functions/flipud.hpp>
#include <nt2/include/functions/run.hpp>
#include <nt2/include/functions/sub2ind.hpp>
#include <nt2/include/functions/ind2sub.hpp>
#include <nt2/include/functions/height.hpp>
#include <nt2/include/functions/enumerate.hpp>
#include <nt2/sdk/meta/as_index.hpp>

namespace nt2 { namespace ext
{
  NT2_FUNCTOR_IMPLEMENTATION( nt2::tag::run_, tag::cpu_
                            , (A0)(State)(Data)(N)
                            , ((node_<A0, nt2::tag::flipud_, N, nt2::container::domain>))
                              (generic_< integer_<State> >)
                              ((unspecified_<Data>))
                            )
  {
    typedef typename boost::dispatch::meta::
            call<nt2::tag::run_ ( typename  boost::proto::result_of::
                                            child_c<A0&, 0>::type
                                , State&, Data&
                                )
                >::type                                        result_type;
    typedef typename meta::as_index<result_type>::type              i_t;
    typedef typename meta::call<nt2::tag::ind2sub_(_2D,i_t)>::type  sub_t;

    BOOST_FORCEINLINE result_type
    operator()(A0 const& a0, State const& p, Data const& t) const
    {
      // Retrieve 2D position from the linear index
      _2D ex = a0.extent();
      sub_t pos = ind2sub(ex, nt2::enumerate<i_t>(p));

      // Compute the flipped start of the enumeration
      pos[0] = height(a0)-pos[0]+1;

      return  nt2::run( boost::proto::child_c<0>(a0)
                      , sub2ind(ex,pos)
                      , t
                      );
    }
  };
} }

#endif

==> pythran-0.6.0.tar.gz: pythran-0.6.0/pythran/nt2/core/functions/common/flipud.hpp <==
//==============================================================================
//         Copyright 2003 - 2011   LASMEA UMR 6602 CNRS/Univ. Clermont II
//         Copyright 2009 - 2011   LRI    UMR 8623 CNRS/Univ Paris Sud XI
//
//          Distributed under the Boost Software License, Version 1.0.
//                 See accompanying file LICENSE.txt or copy at
//                     http://www.boost.org/LICENSE_1_0.txt
//==============================================================================
#ifndef NT2_CORE_FUNCTIONS_COMMON_FLIPUD_HPP_INCLUDED
#define NT2_CORE_FUNCTIONS_COMMON_FLIPUD_HPP_INCLUDED

#include <nt2/core/functions/flipud.hpp>
#include <nt2/include/functions/run.hpp>
#include <nt2/include/functions/dec.hpp>
#include <nt2/core/utility/as_subscript.hpp>
#include <nt2/core/utility/as_index.hpp>
#include <nt2/include/functions/height.hpp>
#include <nt2/include/functions/enumerate.hpp>
#include <nt2/sdk/meta/as_index.hpp>

namespace nt2 { namespace ext
{
  NT2_FUNCTOR_IMPLEMENTATION( nt2::tag::run_, tag::cpu_
                            , (A0)(State)(Data)(N)
                            , ((node_ < A0, nt2::tag::flipud_
                                      , N , nt2::container::domain
                                      >
                              ))
                              (generic_< integer_<State> >)
                              ((unspecified_<Data>))
                            )
  {
    typedef typename Data::type                                     result_type;

    BOOST_FORCEINLINE result_type
    operator()(A0 const& a0, State const& p, Data const& t) const
    {
      typedef typename meta::as_index<result_type>::type                i_t;
      typedef typename result_of::as_subscript<_2D,i_t>::type         sub_t;

      // Retrieve 2D position from the linear index
      _2D ex = a0.extent();
      sub_t pos = as_subscript(ex, nt2::enumerate<i_t>(p));

      // Compute the flipped start of the enumeration
      pos[0] = dec(height(a0)-pos[0]);

      return  nt2::run( boost::proto::child_c<0>(a0)
                      , as_index(ex,pos)
                      , t
                      );
    }
  };
} }

#endif
